function ProgramFilesX86Path{
	#"${Env:ProgramFiles(x86)}" is defined only on x64 machines
	if ("${Env:ProgramFiles(x86)}")
	{
		return "${Env:ProgramFiles(x86)}"
	}
	else
	{
		return "${Env:ProgramFiles}"
	}
}

Set-Variable -Name UiPathDefaulLocation -Option Constant -Value "$(ProgramFilesX86Path)\UiPath\Studio"
$uiPathLocation = $UiPathDefaulLocation


function Set-UiPathLocation([string]$value)
{
	if($value)
	{
		$script:uiPathLocation = $value
	}
}

function Get-UiPathLocation {
	return $script:uiPathLocation
}

function Log()
{
	param(
		[string]$message,
		[ValidateSet("Info", "Warning", "Error", "Verbose")]
		[string]$level = "Info",
		[string]$foregroundColor = "White"
	)
	$dt = [DateTime]::Now
	$date = [string]::Format("{0:dd.MM.yyyy HH:mm:ss}",$dt) + ":"

	switch ($level) {
		"Info" {
			Write-Host "$date $message" -ForegroundColor $foregroundColor
		}
		"Warning" {
			Write-Warning "$date $message"
		}
		"Error" {
			Write-Error "$date $message"
		}
		"Verbose" {
			Write-Verbose "$date $message"
		}
	}
}

function Stop-ServiceWithLog()
{
	param(
		[string]$name
	)
	Log -message "Stopping service: $name"
	Stop-Service -Name $name -ErrorAction SilentlyContinue
}

function Stop-Process-WithLog()
{
	param(
		[string]$name
	)
	Log -message "Stopping process: $name"
	Stop-Process -Name $name -ErrorAction SilentlyContinue
}

function Start-Logman()
{
	param(
		[string]$logsFolder,
		[string]$traceLevel
	)
	$logFile = "$logsFolder\UiPath.etl"
	Log -message "Enabling logman UiPathTrace"
	& logman create trace -cnf 00:00:00 -max 50 UiPathTrace -o "$logFile" -p `{204DE76A-126E-4673-B349-5A5D11D180D7`} 0x01 $traceLevel | Out-Null
	Log -message "Starting logman UiPathTrace"
	& logman start UiPathTrace | Out-Null
}
function Stop-Logman()
{
	Log -message "Stopping logman UiPathTrace"
	& logman stop UiPathTrace | Out-Null
	Log -message "Deleting logman UiPathTrace"
	& logman delete UiPathTrace | Out-Null
}

function NotEmpty()
{
    param (
    [string] $var)

	return ($var -ne "") -and ($var -ne $null);
}

function New-Folder()
{
	param(
		[string]$folderPath,
		[bool]$clear
	)

	if(!(Test-Path $folderPath)) {
		New-Item -ItemType Directory -Force -Path "$folderPath" | Out-Null
	}

	if($clear)
	{
		Get-ChildItem -Path $folderPath | ForEach-Object { Remove-Item -Path $_.FullName -Recurse -Force }
	}
}

function Clear-ETW-Application()
{
	Log -message "Clear all from Application event log"
	Clear-EventLog -LogName Application
}


function Invoke-DownloadMsiFromUrl()
{
	param(
		[string] $url,
		[string] $downloadPath
	)

	Log -message "Start Invoke-DownloadMsiFromUrl"
	Log -message "Downloading from $url to $downloadPath"

	(New-Object System.Net.WebClient).DownloadFile($url, $downloadPath)

	Log -message "End Invoke-DownloadMsiFromUrl"
}


function Invoke-Install()
{
	param(
		[Parameter(Mandatory=$true)]
		[string] $logsFolder,
		[Parameter(Mandatory=$true)]
		[bool] $uninstallPlatform,
		[Parameter(Mandatory=$true)]
		[bool] $installPlatform,
		[string] $platformMsiPath,
		[string] $licenseCode,
		[string] $features,
		[string] $studioInstallPath
	)

	Log -message "Start Invoke-Install"

	if($uninstallPlatform)
	{
		Uninstall-Platform -logsFile "$logsFolder\platformUninstallExisting.log"
	}

	if($installPlatform)
	{
		Install-Platform  -studioInstallPath $studioInstallPath `
			-logsFile "$logsFolder\platformInstall.log" `
			-platformMsiPath $platformMsiPath `
			-licenseCode $licenseCode `
			-features $features
	}

	Log -message "End Invoke-Install"
}


function Uninstall-Platform()
{
	param(
		[string]$logsFile,
		[string]$platformMsiPath
	)
	Log -message "------Starting Uninstall"

	if(![string]::IsNullOrEmpty($platformMsiPath))
	{
		Log -message "Uninstalling product from $platformMsiPath"
		$argumentList = "/qn /x ""$platformMsiPath"" /L*V $logsFile /norestart"
	}
	else
	{
		Log -message "Determining IdentifyingNumber for installed 'UiPath' product"
		$platformGuid = Get-UiPathIdentifyingNumber
		if($platformGuid) {
			Log -message "Uninstalling product with IdentifyingNumber $platformGuid"
			$argumentList = "/X $platformGuid /q /L*V $logsFile /norestart"
		} else {
			Log -message "No application found for uninstalling"
		}
	}

	if(![string]::IsNullOrEmpty($argumentList))
	{
		Log -message "msiexe $argumentList"
		$proc = Start-Process -FilePath msiexec -ArgumentList $argumentList -Wait -PassThru

		if($proc.ExitCode -ne 0){
			Write-Error "Uninstall failed. Check $logsFile"
			throw "Uninstall failed. Check $logsFile"
		}
	}

	Log -message "------Finished Uninstall"
}

function Get-UiPathIdentifyingNumber {
	return (Get-WmiObject -Class Win32_Product -Filter "Name Like '%UiPath%'").IdentifyingNumber
}
function Install-Platform()
{
	param(
		[string]$logsFile,
		[string]$platformMsiPath,
		[string]$licenseCode,
		[string]$features,
		[string]$studioInstallPath
	)

	Log -message "------Start installing Studio"
	Log -message "Installing product from $platformMsiPath"

	$licenseArgument=""
	if($licenseCode) {
		$licenseArgument = "CODE=$licenseCode"
	}
	if($studioInstallPath) {
		if(-Not(Test-Path $studioInstallPath))
		{
			New-Item $studioInstallPath -Type Directory
		}
		$installLocation = "APPLICATIONFOLDER=""$studioInstallPath"""
		Set-UiPathLocation -value "$studioInstallPath"
	}

	$argumentList = "/qn /i ""$platformMsiPath"" $features $licenseArgument $installLocation /L*V $logsFile"
	Log -message "msiexec $argumentList"
	$proc = Start-Process -FilePath msiexec -ArgumentList $argumentList -Wait -PassThru

	if($proc.ExitCode -ne 0)
	{
		Write-Error "Install failed. Check $logsFile"
		throw "Install failed. Check $logsFile"
	}

	$uiRobotPath = "$(Get-UiPathLocation)\UiRobot.exe"
	if(-Not (Test-Path $uiRobotPath))
	{
		Write-Error "Install failed. $uiRobotPath does not exist. Check $logsFile"
		throw "Install failed. $uiRobotPath does not exist. Check $logsFile"
	}

	$regUtilPath = "$(Get-UiPathLocation)\UiPath\Regutil.exe"
	Log -message "$regUtilPath get-info"
	& $regUtilPath get-info

	Log -message "------Finished installing Studio"
}

