
function Deploy () {
    param(
        [Parameter(Mandatory = $true)]
        [String] $orchestratorAddress,
        [Parameter(Mandatory = $true)]
        [ValidateSet("AD", "UserPass")]
        [String] $authType,
        [Parameter(Mandatory = $true)]
        [String] $tenant,
        [Parameter(Mandatory = $false)]
        [String] $username,
        [Parameter(Mandatory = $false)]
        [String] $password,
        [Parameter(Mandatory = $true)]
        [String] $packagePath
    )

    <#
    .SYNOPSIS
        Deploys a package to Uipath Orchestrator
    .PARAMETER orchestratorAddress
        UiPath Orchestrator https address
    .PARAMETER authType
        Authentication type for orchestrator(Username&Password / AD)
    .PARAMETER tenant
        Tenant to be used for login
    .PARAMETER username
        Orchestrator username
    .PARAMETER password
        User password
    .PARAMETER packagePath
        Location that contains one or more packages. If the path is a folder it will upload all packages in that location. If it's a file will only upload that package
    #>

    Log -message "Deploying $packagePath to $orchestratorAddress"
    if ($authType -eq "AD") {
        Get-UiPathAuthToken -URL $orchestratorAddress -WindowsCredentials -TenantName $tenant -Session
    }
    else {
        Get-UiPathAuthToken -URL $orchestratorAddress -TenantName $tenant -Username $username -Password $password -Session
    }

    if ( Test-Path -Path $packagePath -PathType Leaf) {
        Add-UiPathPackage -PackageFile $packagePath
        Log -message "Package $packagePath successfully added"
    }
    else {
        #upload all *.nupkg except *.symbols.nupkg"
        $packageFiles = (Get-ChildItem $packagePath | Where {($_.extension -eq ".nupkg") -and ($_ -notlike "*.symbols.nupkg") }).FullName
        Log -message "Found $($packageFiles.Count) packages to upload to $orchestratorAddress"
        if ($packageFiles.Count -gt 0) {
            $packageFiles | foreach { Write-Host "Adding $_"; Add-UiPathPackage -PackageFile "$_"; Log -message "Package $_ successfully added" }
        }
    }
}



function InstallAndPack($platformVersion, $msiLocation, $setupMsiName, $uninstallOption, $licenseCode, $uninstallAfter, $projectJsonPath, $packageVersion,
    $logsFolder, $outputFolder, $studioInstallPath) {
    <#
.SYNOPSIS
    Creates a new package from given $projectJsonPath using UiPathStudio
.DESCRIPTION
    Takes as input an UiPath project source code and creates a nuget package ready to be shipped in Orchestrator and further deployed on Robot machines.
    It calls -pack method from UiPathRobot.exe
    Running the above command doesn't require the Robot installation to be licensed.
    Its output should be added as an artifact of the Azure DevOps build pipeline.
.PARAMETER platformVersion
    Specifies the platform version to use for creating the package
.PARAMETER msiLocation
    Location from where to download the UiPathStudio msi
.PARAMETER setupMsiName
    Name of the file to search when downloading from BLOB
.PARAMETER uninstallOption
    ["never","ifany","ifdifferent"] - options to use if an existing version is already installed
.PARAMETER licenseCode
    License code to use for installing UiPathStudio
.PARAMETER uninstallAfter
    Option to leave the installed UiPathStudio on the machine
.PARAMETER projectJsonPath
    Path to project json that should be packed in a nuget package
.PARAMETER packageVersion
    Version to use when creating the package. If empty, an autogenerated version will be used
.PARAMETER logsFolder
    Logs folder
.PARAMETER outputFolder
    Location where the created package will be saved
#>

    Install -platformVersion $platformVersion `
        -msiLocation $msiLocation `
        -setupMsiName $setupMsiName `
        -uninstallOption $uninstallOption `
        -licenseCode $licenseCode `
        -logsFolder $logsFolder `
        -studioInstallPath $studioInstallPath

    Pack -projectJsonPath $projectJsonPath `
        -packageVersion $packageVersion `
        -rootFolder $rootFolder `
        -logsFolder $logsFolder `
        -outputFolder $outputFolder `
        -studioInstallPath $studioInstallPath


    if ($uninstallAfter) {
        Uninstall-Platform
    }

}

function Install ($platformVersion, $msiLocation, $setupMsiName, $uninstallOption, $licenseCode, $logsFolder, $studioInstallPath) {
    <#
    .SYNOPSIS
        Creates a new package from given $projectJsonPath using UiPathStudio
    .DESCRIPTION
        Takes as input an UiPath project source code and creates a nuget package ready to be shipped in Orchestrator and further deployed on Robot machines.
        It calls -pack method from UiPathRobot.exe
        Running the above command doesn't require the Robot installation to be licensed.
        Its output should be added as an artifact of the Azure DevOps build pipeline.
    .PARAMETER platformVersion
        Specifies the platform version to use for creating the package
    .PARAMETER msiLocation
        Location from where to download the UiPathStudio msi
    .PARAMETER setupMsiName
        Name of the file to search when downloading from BLOB
    .PARAMETER uninstallOption
        ["never","ifany","ifdifferent"] - options to use if an existing version is already installed
    .PARAMETER licenseCode
        License code to use for installing UiPathStudio
    .PARAMETER logsFolder
        Logs folder
#>

    $uninstallPlatform = $false
    $installPlatform = $true
    $installedObject = Get-UiPathStudioRegObject
    switch ($uninstallOption) {
        "never" {
            Log -message "No uninstall required"
            if ($installedObject) {
                Log -message "Platform installed"
                $installPlatform = $false
            }
            else {
                Log -level Warning -message "Platform not  installed. Installation required!"
            }

            break
        }
        "ifany" {
            $uninstallPlatform = $true
        }
        "ifdifferent" {
            if ($installedObject.DisplayVersion -ne $platformVersion) {
                Log -message "Found different platform version"
                $uninstallPlatform = $true
            }
            else {
                $installPlatform = $false
            }
        }
        Default {
            Log -message "Uninstall option [$uninstallOption] not found"
        }
    }


    if (!$msiLocation) {
        # we need an updated version.json file in the blob
        # $versionAndMsiName = Get-VersionAndMsiName($platformVersion)
        # $platformVersion = $versionAndMsiName[0]
        # $setupMsiName = $versionAndMsiName[1]
    }
    else {
        $platformVersion = $null
        $lastIndex = $msiLocation.LastIndexOf("/")
        $setupMsiName = $msiLocation.Substring($lastIndex + 1 , $msiLocation.Length - $lastIndex - 1)
    }

    Write-Host "Platform version: $platformVersion"
    Write-Host "Setup msi name: $setupMsiName"

    Stop-ServiceWithLog -name "UiRobotSvc" -ignoreFailure $true
	Stop-Process-WithLog -name "UiPath.Executor.exe"
	Stop-Process-WithLog -name "UiRobot.exe"

    Log -message "Uninstall options: $uninstallOption -> Uninstall platform: $uninstallPlatform"
    Log -message "Install platform: $installPlatform"

    if ($installPlatform) {
        [string]$msiPath = Download-UiPathPlatform `
            -platformVersion $platformVersion `
            -setupMsiName $setupMsiName `
            -msiLocation $msiLocation
    }

    # Install platform
    Invoke-Install `
        -logsFolder $logsFolder `
        -uninstallPlatform $uninstallPlatform `
        -installPlatform $installPlatform `
        -platformMsiPath $msiPath  `
        -licenseCode $licenseCode `
        -studioInstallPath $studioInstallPath `
        -features "ADDLOCAL=DesktopFeature,Studio,Robot,RegisterService,Packages"

}

function Pack($projectJsonPath, $packageVersion, $studioInstallPath, $outputFolder) {
    <#
    .SYNOPSIS
        Creates a new package from given $projectJsonPath using UiPathStudio
    .DESCRIPTION
        Takes as input an UiPath project source code and creates a nuget package ready to be shipped in Orchestrator and further deployed on Robot machines.
        It calls -pack method from UiPathRobot.exe
        Running the above command doesn't require the Robot installation to be licensed.
        Its output should be added as an artifact of the Azure DevOps build pipeline.
    .PARAMETER projectJsonPath
        Path to project json that should be packed in a nuget package
    .PARAMETER packageVersion
        Version to use when creating the package. If empty, an autogenerated version will be used
    .PARAMETER studioInstallPath
        Location where UiPath Studio is installed
    .PARAMETER outputFolder
        Location where the created package will be saved
#>
    if (-Not (Test-Path $outputFolder)) {
        New-Item -ItemType Directory -Force -Path $outputFolder
    }
    if ($studioInstallPath) {
        Set-UiPathLocation -value $studioInstallPath
    }

    if (Test-Path $projectJsonPath -PathType Leaf) {
        $packResult = PackProjectFile -projectPath $projectJsonPath -version $packageVersion -outputFolder $outputFolder
    }
    elseif (Test-Path $projectJsonPath -Type Container) {
        if (Test-Path "$projectJsonPath\project.json" -Type Leaf) {
            $packResult = PackProjectFile -projectPath "$projectJsonPath\project.json" -version $packageVersion -outputFolder "$outputFolder"
        }
        else {
            $subFolders = Get-ChildItem -Path "$projectJsonPath" -Directory | Select-Object FullName
            $subFolders | ForEach   {
                                        If (Test-Path "$($_.FullName)\project.json" -Type Leaf) {
                                            $packResult += PackProjectFile -projectPath "$($_.FullName)\project.json" -version $packageVersion  -outputFolder "$outputFolder"
                                            $packResult += "`n"
                                        }
                                    }
        }
    }

    if (-Not($packResult)) {
        Throw-TerminatingError -target "PackResult"  -message "Pack command generated no result. Might be a problem with the specified parameters.[$projectJsonPath | $packageVersion | $studioInstallPath | $outputFolder]"
    }

    Log -message "Pack result: $packResult"
}

function Throw-TerminatingError {
    [CmdletBinding()]
    Param(
        [String]$message,
        [String] $target
    )

    [System.Exception]$exception = "$message"
    [System.String]$errorID = 'UiPathPackage-Module'
    [System.Management.Automation.ErrorCategory]$errorCategory =  [System.Management.Automation.ErrorCategory]::InvalidOperation
    [System.Object]$target = $target
    $errorRecord = New-Object System.Management.Automation.ErrorRecord($exception, $errorID, $errorCategory, $target )
    $PSCmdlet.ThrowTerminatingError($errorRecord)
 }

function PackProjectFile($projectPath, $version, $outputFolder) {
    if ([string]::IsNullOrEmpty($version)) {
        Log -message "Running pack with the arguments: -pack $projectPath -o $outputFolder"
        $packResult = & "$(Get-UiPathLocation)\UiRobot.exe" -pack "$projectPath" -o "$outputFolder"
    }
    else {
        Log -message "Running pack with the arguments: -pack $projectPath -o $outputFolder -v $version"
        $packResult = & "$(Get-UiPathLocation)\UiRobot.exe" -pack "$projectPath" -o "$outputFolder" -v $version
    }
    return $packResult
}
function Download-UiPathPlatform($platformVersion, $setupMsiName, $msiLocation) {

    if($env:Agent_TempDirectory) {
        $tempFolder = Join-Path $env:Agent_TempDirectory "r$(Get-Date -f 'ssfff')"
    }
    else {
        $tempFolder = Join-Path $PSScriptRoot "r$(Get-Date -f 'ssfff')"
    }
    $msiDownloadFolder = Join-Path $tempFolder $setupArtifactName
    $msiDownloadPath = Join-Path $msiDownloadFolder $setupMsiName

    if (-Not (Test-Path -Path $msiDownloadFolder)) {
        New-Item -Path $msiDownloadFolder -ItemType Directory | Out-Null
    }

    $url = $msiLocation

    if (!$url) {
        $url = if ($platformVersion) {
            "https://download.uipath.com/versions/$platformVersion/$setupMsiName"
        }
        else {
            "https://download.uipath.com/$setupMsiName"
        }
    }

    Log -message "Downloading Setup from $url to $msiDownloadPath"

    Invoke-DownloadMsiFromUrl `
        -url $url `
        -downloadPath $msiDownloadPath

    return $msiDownloadPath
}

function Get-UiPathStudioRegObject {
    <#
    .SYNOPSIS
        Get an object containing details about the UiPath Studio installed instance
    .DESCRIPTION
        The returned object will contain: DisplayName, DisplayVersion, Publisher, InstallDate, UninstallString
    #>

    return Get-ItemProperty HKLM:\Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\* | Where-object DisplayName -like "*UiPath Studio*" | Select-Object DisplayName, DisplayVersion, Publisher, InstallDate, UninstallString
}
